@@ -1,14 +1,4 @@
-# Licensed under the Apache License, Version 2.0 (the "License");
-# you may not use this file except in compliance with the License.
-# You may obtain a copy of the License at
-#
-#     http://www.apache.org/licenses/LICENSE-2.0
-#
-# Unless required by applicable law or agreed to in writing, software
-# distributed under the License is distributed on an "AS IS" BASIS,
-# WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
-# See the License for the specific language governing permissions and
-# limitations under the License.
+# SPDX-License-Identifier: Apache-2.0
 
 import mock
 import pytest
@@ -38,6 +28,7 @@ LAST_MODIFIED_AT = date.today().isoformat()
 DESCRIPTION = 'test description'
 TAG_NAME = 'test'
 TAGS = [TAG_NAME]
+VERSION = str(uuid.uuid4())
 
 # NAMESPACE
 NAMESPACE_NAME = 'test-namespace'
@@ -202,6 +193,33 @@ FAILED = {
     'args': None
 }
 
+# DATASET VERSIONS
+DB_TABLE_VERSION = {
+    'id': DB_TABLE_ID,
+    'type': DatasetType.DB_TABLE,
+    'name': DB_TABLE_NAME,
+    'physicalName': DB_TABLE_PHYSICAL_NAME,
+    'createdAt': NOW_AS_ISO,
+    'version': VERSION,
+    'namespace': NAMESPACE_NAME,
+    'sourceName': SOURCE_NAME,
+    'fields': FIELDS,
+    'tags': [],
+    'description': DESCRIPTION,
+    'createdByRun': COMPLETED
+}
+
+# TAGS
+TAG_NAME2 = "second_tag"
+TAG_DESCRIPTION2 = "second tag description"
+TAG = {
+    'name': TAG_NAME,
+    'description': DESCRIPTION
+}
+TAG_LIST = {
+    'tags': [{TAG_NAME: DESCRIPTION}, {TAG_NAME2: TAG_DESCRIPTION2}]
+}
+
 
 @pytest.fixture
 def client():
@@ -416,6 +434,71 @@ def test_get_dataset(mock_get, client):
     )
 
 
+@mock.patch('requests.get')
+def test_get_dataset_version(mock_get, client):
+    mock_get.return_value.status_code.return_value = HTTPStatus.OK
+    mock_get.return_value.json.return_value = DB_TABLE_VERSION
+
+    dataset_version = client.get_dataset_version(
+        namespace_name=NAMESPACE_NAME,
+        dataset_name=DB_TABLE_NAME,
+        version=VERSION
+    )
+
+    assert dataset_version['id'] == DB_TABLE_ID
+    assert dataset_version['type'] == DatasetType.DB_TABLE
+    assert dataset_version['name'] == DB_TABLE_NAME
+    assert dataset_version['physicalName'] == DB_TABLE_PHYSICAL_NAME
+    assert dataset_version['version'] == VERSION
+    assert dataset_version['sourceName'] == SOURCE_NAME
+    assert dataset_version['fields'] == FIELDS
+    assert dataset_version['description'] == DESCRIPTION
+    assert dataset_version['createdByRun'] == COMPLETED
+
+    mock_get.assert_called_once_with(
+        url=client._url(
+            '/namespaces/{0}/datasets/{1}/versions/{2}',
+            NAMESPACE_NAME, DB_TABLE_NAME, VERSION
+        ),
+        params=mock.ANY,
+        headers=mock.ANY,
+        timeout=mock.ANY
+    )
+
+
+@mock.patch('requests.get')
+def test_list_dataset_versions(mock_get, client):
+    mock_get.return_value.status_code.return_value = HTTPStatus.OK
+    mock_get.return_value.json.return_value = [DB_TABLE_VERSION]
+
+    dataset_versions = client.list_dataset_versions(
+        NAMESPACE_NAME, DB_TABLE_NAME
+    )
+
+    assert dataset_versions[0]['id'] == DB_TABLE_ID
+    assert dataset_versions[0]['type'] == DatasetType.DB_TABLE
+    assert dataset_versions[0]['name'] == DB_TABLE_NAME
+    assert dataset_versions[0]['physicalName'] == DB_TABLE_PHYSICAL_NAME
+    assert dataset_versions[0]['version'] == VERSION
+    assert dataset_versions[0]['sourceName'] == SOURCE_NAME
+    assert dataset_versions[0]['fields'] == FIELDS
+    assert dataset_versions[0]['description'] == DESCRIPTION
+    assert dataset_versions[0]['createdByRun'] == COMPLETED
+
+    mock_get.assert_called_once_with(
+        url=client._url(
+            '/namespaces/{0}/datasets/{1}/versions',
+            NAMESPACE_NAME, DB_TABLE_NAME
+        ),
+        params={
+            'limit': DEFAULT_LIMIT,
+            'offset': DEFAULT_OFFSET
+        },
+        headers=mock.ANY,
+        timeout=mock.ANY
+    )
+
+
 @mock.patch('requests.get')
 def test_list_datasets(mock_get, client):
     mock_get.return_value.status_code.return_value = HTTPStatus.OK
@@ -775,3 +858,47 @@ def test_mark_job_run_as_failed(mock_post, client):
         json=None,
         timeout=mock.ANY
     )
+
+
+@mock.patch('requests.put')
+def test_create_tag(mock_put, client):
+    mock_put.return_value.status_code.return_value = HTTPStatus.OK
+    mock_put.return_value.json.return_value = TAG
+
+    tag = client.create_tag(TAG_NAME, DESCRIPTION)
+
+    assert tag['name'] == TAG_NAME
+    assert tag['description'] == DESCRIPTION
+
+    mock_put.assert_called_once_with(
+        url=client._url(
+            '/tags/{0}', TAG_NAME
+        ),
+        headers=mock.ANY,
+        json={
+            "description": DESCRIPTION
+        },
+        timeout=mock.ANY
+    )
+
+
+@mock.patch('requests.get')
+def test_list_tags(mock_get, client):
+    mock_get.return_value.status_code.return_value = HTTPStatus.OK
+    mock_get.return_value.json.return_value = TAG_LIST
+
+    tags = client.list_tags(limit=DEFAULT_LIMIT, offset=DEFAULT_OFFSET)
+
+    assert tags == TAG_LIST
+
+    mock_get.assert_called_once_with(
+        url=client._url(
+            '/tags'
+        ),
+        headers=mock.ANY,
+        params={
+            'limit': DEFAULT_LIMIT,
+            'offset': DEFAULT_OFFSET
+        },
+        timeout=mock.ANY
+    )
